<?php

namespace FiloBlu\ExtInventory\Plugin\Api\OrderRepositoryInterface;

use Exception;
use FiloBlu\ExtInventory\Api\InventoryFbOrderItemSourceRepositoryInterface;
use FiloBlu\ExtInventory\Helper\Data;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Sales\Api\Data\OrderExtensionFactory;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\Data\OrderItemExtension;
use Magento\Sales\Api\Data\OrderItemExtensionFactory;
use Magento\Sales\Api\Data\OrderItemInterface;
use Magento\Sales\Api\OrderRepositoryInterface;

/**
 * Class AfterGet
 * @package FiloBlu\ExtInventory\Plugin\Api\OrderRepositoryInterface
 */
class AfterGet
{
    /**
     * @var OrderExtensionFactory
     */
    protected $orderExtensionFactory;

    /**
     * @var OrderItemExtensionFactory
     */
    protected $orderItemExtensionFactory;
    /**
     * @var InventoryFbOrderItemSourceRepositoryInterface
     */
    private $inventoryFbOrderItemSourceRepository;
    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;
    /**
     * @var Data
     */
    private $helperData;

    /**
     * AfterGet constructor.
     * @param OrderExtensionFactory $orderExtensionFactory
     * @param OrderItemExtensionFactory $orderItemExtensionFactory
     * @param InventoryFbOrderItemSourceRepositoryInterface $inventoryFbOrderItemSourceRepository
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param Data $helperData
     */
    public function __construct(
        OrderExtensionFactory                         $orderExtensionFactory,
        OrderItemExtensionFactory                     $orderItemExtensionFactory,
        InventoryFbOrderItemSourceRepositoryInterface $inventoryFbOrderItemSourceRepository,
        SearchCriteriaBuilder                         $searchCriteriaBuilder,
        Data                                          $helperData
    ) {
        $this->orderExtensionFactory = $orderExtensionFactory;
        $this->orderItemExtensionFactory = $orderItemExtensionFactory;
        $this->inventoryFbOrderItemSourceRepository = $inventoryFbOrderItemSourceRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->helperData = $helperData;
    }

    /**
     * Add Data to order
     *
     * @param OrderRepositoryInterface $subject
     * @param OrderInterface $resultOrder
     * @return OrderInterface
     */
    public function afterGet(
        OrderRepositoryInterface $subject,
        OrderInterface           $resultOrder
    ) {

        $extensionAttributes = $resultOrder->getExtensionAttributes();

        if (!$extensionAttributes) {
            $extensionAttributes = $this->orderExtensionFactory->create();
            $resultOrder->setExtensionAttributes($extensionAttributes);
        }

        $stockType = $this->helperData->getInventoryType($resultOrder->getStoreId());
        $extensionAttributes->setFilobluInventoryType($stockType);

        return $this->getFiloBluOrderItems($resultOrder);
    }

    /**
     * Get gift message for items of order
     *
     * @param OrderInterface $order
     * @return OrderInterface
     */
    protected function getFiloBluOrderItems(OrderInterface $order)
    {
        $orderItems = $order->getItems();

        if (null === $orderItems || empty($orderItems)) {
            return $order;
        }

        /** @var OrderItemInterface $orderItem */
        foreach ($orderItems as $orderItem) {
            $extensionAttributes = $orderItem->getExtensionAttributes();
            if ($extensionAttributes === null) {
                continue;
            }

            try {
                $this->searchCriteriaBuilder->addFilter('order_item_id', $orderItem->getItemId());
                $filoBluOrderItem = $this->inventoryFbOrderItemSourceRepository
                    ->getList(
                        $this->searchCriteriaBuilder->addFilter('order_item_id', $orderItem->getItemId())
                        ->create()
                    );
                $filoBluOrderItems = $filoBluOrderItem->getItems();
                /** @var OrderItemExtension $orderItemExtension */
                $orderItemExtension = $extensionAttributes ?: $this->orderItemExtensionFactory->create();
                $orderItemExtension->setFilobluOrderItem($filoBluOrderItems);
                $orderItem->setExtensionAttributes($orderItemExtension);
            } catch (Exception $exception) {
                continue;
            }
        }

        return $order;
    }
}
